@@ -2,6 +2,7 @@ use std::collections::HashMap;
 
 use ratatui::{
     backend::Backend,
+    crossterm::event::{KeyCode, KeyEvent},
     layout::{Constraint, Layout, Rect},
     style::{Modifier, Style, Stylize},
     text::Line,
@@ -12,7 +13,7 @@ use ratatui::{
 use crate::{
     color::{ColorTheme, GraphColorSet},
     config::{CoreConfig, CursorType, UiConfig},
-    event::{AppEvent, Receiver, Sender, UserEvent},
+    event::{AppEvent, Receiver, Sender, UserEvent, UserEventWithCount},
     external::copy_to_clipboard,
     git::Repository,
     graph::{CellWidthType, Graph, GraphImageManager},
@@ -39,10 +40,14 @@ pub struct App<'a> {
     status_line: StatusLine,
 
     keybind: &'a KeyBind,
+    core_config: &'a CoreConfig,
     ui_config: &'a UiConfig,
     color_theme: &'a ColorTheme,
     image_protocol: ImageProtocol,
     tx: Sender,
+
+    numeric_prefix: String,
+    view_area: Rect,
 }
 
 impl<'a> App<'a> {
@@ -95,10 +100,13 @@ impl<'a> App<'a> {
             status_line: StatusLine::None,
             view,
             keybind,
+            core_config,
             ui_config,
             color_theme,
             image_protocol,
             tx,
+            numeric_prefix: String::new(),
+            view_area: Rect::default(),
         }
     }
 }
@@ -130,15 +138,44 @@ impl App<'_> {
                         }
                     }
 
-                    match self.keybind.get(&key) {
+                    let user_event = self.keybind.get(&key);
+
+                    if let Some(UserEvent::Cancel) = user_event {
+                        if !self.numeric_prefix.is_empty() {
+                            // Clear numeric prefix and cancel the event
+                            self.numeric_prefix.clear();
+                            continue;
+                        }
+                    }
+
+                    match user_event {
                         Some(UserEvent::ForceQuit) => {
                             self.tx.send(AppEvent::Quit);
                         }
                         Some(ue) => {
-                            self.view.handle_event(*ue, key);
+                            let event_with_count =
+                                process_numeric_prefix(&self.numeric_prefix, *ue, key);
+                            self.view.handle_event(event_with_count, key);
+                            self.numeric_prefix.clear();
                         }
                         None => {
-                            self.view.handle_event(UserEvent::Unknown, key);
+                            if let StatusLine::Input(_, _, _) = self.status_line {
+                                // In input mode, pass all key events to the view
+                                // fixme: currently, the only thing that processes key_event is searching the list,
+                                //        so this probably works, but it's not the right process...
+                                self.numeric_prefix.clear();
+                                self.view.handle_event(
+                                    UserEventWithCount::from_event(UserEvent::Unknown),
+                                    key,
+                                );
+                            } else if let KeyCode::Char(c) = key.code {
+                                // Accumulate numeric prefix
+                                if c.is_ascii_digit()
+                                    && (c != '0' || !self.numeric_prefix.is_empty())
+                                {
+                                    self.numeric_prefix.push(c);
+                                }
+                            }
                         }
                     }
                 }
@@ -157,6 +194,15 @@ impl App<'_> {
                 AppEvent::ClearDetail => {
                     self.clear_detail();
                 }
+                AppEvent::OpenUserCommand(n) => {
+                    self.open_user_command(n);
+                }
+                AppEvent::CloseUserCommand => {
+                    self.close_user_command();
+                }
+                AppEvent::ClearUserCommand => {
+                    self.clear_user_command();
+                }
                 AppEvent::OpenRefs => {
                     self.open_refs();
                 }
@@ -206,6 +252,8 @@ impl App<'_> {
         let [view_area, status_line_area] =
             Layout::vertical([Constraint::Min(0), Constraint::Length(2)]).areas(f.area());
 
+        self.update_state(view_area);
+
         self.view.render(f, view_area);
         self.render_status_line(f, status_line_area);
     }
@@ -214,7 +262,14 @@ impl App<'_> {
 impl App<'_> {
     fn render_status_line(&self, f: &mut Frame, area: Rect) {
         let text: Line = match &self.status_line {
-            StatusLine::None => "".into(),
+            StatusLine::None => {
+                if self.numeric_prefix.is_empty() {
+                    Line::raw("")
+                } else {
+                    Line::raw(self.numeric_prefix.as_str())
+                        .fg(self.color_theme.status_input_transient_fg)
+                }
+            }
             StatusLine::Input(msg, _, transient_msg) => {
                 let msg_w = console::measure_text_width(msg.as_str());
                 if let Some(t_msg) = transient_msg {
@@ -266,6 +321,10 @@ impl App<'_> {
 }
 
 impl App<'_> {
+    fn update_state(&mut self, view_area: Rect) {
+        self.view_area = view_area;
+    }
+
     fn open_detail(&mut self) {
         if let View::List(ref mut view) = self.view {
             let commit_list_state = view.take_list_state();
@@ -308,6 +367,108 @@ impl App<'_> {
         }
     }
 
+    fn open_user_command(&mut self, user_command_number: usize) {
+        if let View::List(ref mut view) = self.view {
+            let commit_list_state = view.take_list_state();
+            let selected = commit_list_state.selected_commit_hash().clone();
+            let (commit, _) = self.repository.commit_detail(&selected);
+            self.view = View::of_user_command_from_list(
+                commit_list_state,
+                commit,
+                user_command_number,
+                self.view_area,
+                self.core_config,
+                self.ui_config,
+                self.color_theme,
+                self.image_protocol,
+                self.tx.clone(),
+            );
+        } else if let View::Detail(ref mut view) = self.view {
+            let commit_list_state = view.take_list_state();
+            let selected = commit_list_state.selected_commit_hash().clone();
+            let (commit, _) = self.repository.commit_detail(&selected);
+            self.view = View::of_user_command_from_detail(
+                commit_list_state,
+                commit,
+                user_command_number,
+                self.view_area,
+                self.core_config,
+                self.ui_config,
+                self.color_theme,
+                self.image_protocol,
+                self.tx.clone(),
+            );
+        } else if let View::UserCommand(ref mut view) = self.view {
+            let commit_list_state = view.take_list_state();
+            let selected = commit_list_state.selected_commit_hash().clone();
+            let (commit, _) = self.repository.commit_detail(&selected);
+            if view.before_view_is_list() {
+                self.view = View::of_user_command_from_list(
+                    commit_list_state,
+                    commit,
+                    user_command_number,
+                    self.view_area,
+                    self.core_config,
+                    self.ui_config,
+                    self.color_theme,
+                    self.image_protocol,
+                    self.tx.clone(),
+                );
+            } else {
+                self.view = View::of_user_command_from_detail(
+                    commit_list_state,
+                    commit,
+                    user_command_number,
+                    self.view_area,
+                    self.core_config,
+                    self.ui_config,
+                    self.color_theme,
+                    self.image_protocol,
+                    self.tx.clone(),
+                );
+            }
+        }
+    }
+
+    fn close_user_command(&mut self) {
+        if let View::UserCommand(ref mut view) = self.view {
+            let commit_list_state = view.take_list_state();
+            let selected = commit_list_state.selected_commit_hash().clone();
+            let (commit, changes) = self.repository.commit_detail(&selected);
+            let refs = self
+                .repository
+                .refs(&selected)
+                .into_iter()
+                .cloned()
+                .collect();
+            if view.before_view_is_list() {
+                self.view = View::of_list(
+                    commit_list_state,
+                    self.ui_config,
+                    self.color_theme,
+                    self.tx.clone(),
+                );
+            } else {
+                self.view = View::of_detail(
+                    commit_list_state,
+                    commit,
+                    changes,
+                    refs,
+                    self.ui_config,
+                    self.color_theme,
+                    self.image_protocol,
+                    self.tx.clone(),
+                );
+            }
+        }
+    }
+
+    fn clear_user_command(&mut self) {
+        if let View::UserCommand(ref mut view) = self.view {
+            view.clear();
+        }
+    }
+
     fn open_refs(&mut self) {
         if let View::List(ref mut view) = self.view {
             let commit_list_state = view.take_list_state();
@@ -342,6 +503,7 @@ impl App<'_> {
             self.image_protocol,
             self.tx.clone(),
             self.keybind,
+            self.core_config,
         );
     }
 
@@ -398,3 +560,47 @@ impl App<'_> {
         }
     }
 }
+
+fn process_numeric_prefix(
+    numeric_prefix: &str,
+    user_event: UserEvent,
+    _key_event: KeyEvent,
+) -> UserEventWithCount {
+    if user_event.is_countable() {
+        let count = if numeric_prefix.is_empty() {
+            1
+        } else {
+            numeric_prefix.parse::<usize>().unwrap_or(1)
+        };
+        UserEventWithCount::new(user_event, count)
+    } else {
+        UserEventWithCount::from_event(user_event)
+    }
+}
+
+#[cfg(test)]
+mod tests {
+    use rstest::rstest;
+
+    use super::*;
+
+    #[rustfmt::skip]
+    #[rstest]
+    #[case("",    UserEvent::NavigateDown, UserEventWithCount::new(UserEvent::NavigateDown, 1))] // no prefix
+    #[case("5",   UserEvent::NavigateUp,   UserEventWithCount::new(UserEvent::NavigateUp, 5))] // with prefix
+    #[case("0",   UserEvent::PageDown,     UserEventWithCount::new(UserEvent::PageDown, 1))] // zero should be converted to 1
+    #[case("42",  UserEvent::ScrollDown,   UserEventWithCount::new(UserEvent::ScrollDown, 42))] // multi-digit number
+    #[case("999", UserEvent::PageDown,     UserEventWithCount::new(UserEvent::PageDown, 999))] // large number
+    #[case("abc", UserEvent::ScrollUp,     UserEventWithCount::new(UserEvent::ScrollUp, 1))] // should fallback to 1
+    #[case("5",   UserEvent::Quit,         UserEventWithCount::new(UserEvent::Quit, 1))] // non-countable event with prefix
+    #[case("",    UserEvent::Confirm,      UserEventWithCount::new(UserEvent::Confirm, 1))] // non-countable event without prefix
+    fn test_process_numeric_prefix(
+        #[case] numeric_prefix: &str,
+        #[case] user_event: UserEvent,
+        #[case] expected: UserEventWithCount,
+    ) {
+        let dummy_key_event = KeyEvent::from(KeyCode::Enter); // KeyEvent is not used in the logic
+        let actual = process_numeric_prefix(numeric_prefix, user_event, dummy_key_event);
+        assert_eq!(actual, expected);
+    }
+}
